
setMethod("gateNodeToFilterObject",
          signature(aNode = "XMLInternalElementNode"), 
          definition = function(aNode, 
                                quiet=TRUE, 
                                stripComp=TRUE) {

    fjVersion    <- getFlowJoVersion(aNode);
    gateType     <- xmlName(aNode);
    ancNodes     <- xmlAncestors(aNode);
    sampInx      <- which(lapply(ancNodes, xmlName) == "Sample")[1];
    ns = c(gating   = "http://www.isac-net.org/std/Gating-ML/v1.5/gating", 
           datatype = "http://www.isac-net.org/std/Gating-ML/v1.5/datatypes");


    # As of FJ v. 7.2.5, FlowJo had scatter gate coordinates at 1/64
    # of their "actual" value.  I think they'll fix this in the future?
    # FIRST, NEED TO FIND WHICH VALUES CORRESPOND TO SSC-A
    # THEN EXTRACT THE CORRESPONDING PxR
    pr1Value     <- as.numeric(xpathApply(ancNodes[[sampInx]], 
                                     "Keywords/Keyword[@name='$P1R']", 
                                     xmlGetAttr, "value")[[1]]);
    
    multiplier = 64;
    if ((compareVersion(fjVersion, "7.4.9") > 0) | (pr1Value < 4096)) { 
       multiplier = 1; 
    }  

    # FIGURE OUT WHAT THE NAME OF THE GATE IS, EG. "Lymphocytes"
    index        <- ancNodes == "Population";
    parentNames  <- unlist(sapply(ancNodes, xmlGetAttr, "name")[index]);
    gateName     <- paste(parentNames[ length(parentNames) ], collapse=":");   # name of this gate
    
#    popNode  = ancs[length(ancs)-1];
#      gateName = xmlGetAttr(popNode[[1]], "gating:id");

           
    # POLYGON GATES
    if (gateType == "PolygonGate") {
      axesNames = unlist(xpathApply(aNode, ".//gating:dimension/datatype:parameter", xmlGetAttr, "data-type:name", namespaces = ns));        
      xPoints = as.numeric(unlist(xpathApply(aNode, ".//gating:vertex/gating:coordinate[1]", xmlGetAttr, "data-type:value", namespaces = ns)));
      yPoints = as.numeric(unlist(xpathApply(aNode, ".//gating:vertex/gating:coordinate[2]", xmlGetAttr, "data-type:value", namespaces = ns)));
      if (compareVersion(fjVersion, "7.4.9") < 0) {
        axesNames = unlist(xpathApply(aNode, ".//Axis", xmlGetAttr, "name"))
        xPoints   = as.numeric(unlist(xpathApply(aNode, ".//Point", xmlGetAttr, "x")))
        yPoints   = as.numeric(unlist(xpathApply(aNode, ".//Point", xmlGetAttr, "y")))
      }
      
      # STRIP OUT THE "Comp-" PART OF NAMES.   THE TRANSFORMATION IS ONLY ON THE VISUALIZATION
      # OF DATA IN FLOWJO GRAPHS AND HAS NO IMPACT ON THE STORED GATING COORDINATES.
      if (stripComp == TRUE) {
        axesNames = gsub("\\bComp-", "", axesNames, perl = TRUE)
      }

      verticies = matrix(c(xPoints, yPoints), ncol = 2, nrow = length(xPoints),
          byrow = FALSE);
      colnames(verticies) = axesNames;
      for (i in 1:length(axesNames)) { 
        if (length(grep("FSC-[AHW]|SSC-[AHW]", axesNames[i])) > 0) {
          verticies[,i] = verticies[,i] * multiplier
        }
      }

      thisGate = polygonGate(filterId = gateName, .gate = verticies)
      thisGate@filterId = gateName
      return(thisGate)
    }
    
    # RECTANGLE AND RANGE GATES
    if ((gateType == "RectangleGate") | (gateType == "RangeGate")) {
      # Normal 2-D "RectangleGates"
      if (xmlSize(aNode) == 1) {
        rangeNames = unlist(xpathApply(aNode, ".//gating:dimension/datatype:parameter", xmlGetAttr,"data-type:name",namespaces = ns));
        rangeMins  = as.numeric(xpathApply(aNode, ".//gating:dimension", xmlGetAttr, "gating:min", namespaces = ns));
        rangeMaxes = as.numeric(xpathApply(aNode, ".//gating:dimension", xmlGetAttr, "gating:max", namespaces = ns));
        if (compareVersion(fjVersion, "7.4.9") < 0) {
          rangeNames = unlist(xpathApply(aNode, ".//Axis", xmlGetAttr, "name"))
          rangeMins  = as.numeric(xmlGetAttr(aNode, "min"));
          rangeMaxes = as.numeric(xmlGetAttr(aNode, "max"));
        }

      # Just a 1-D "RangeGate"
      } else {
#      	print(xmlSize(aNode));
        rangeNames = unlist(xpathApply(aNode, ".//gating:dimension/datatype:parameter", xmlGetAttr, "data-type:name",namespaces = ns));
        rangeMins  = as.numeric(unlist(xpathApply(aNode, ".//gating:dimension", xmlGetAttr, "gating:min", namespaces = ns)));
        rangeMaxes = as.numeric(unlist(xpathApply(aNode, ".//gating:dimension", xmlGetAttr, "gating:max", namespaces = ns)));
        if (compareVersion(fjVersion, "7.4.9") < 0) {
          rangeNames = unlist(xpathApply(aNode, ".//RangeGate/Axis", xmlGetAttr, "name"))
          rangeMins  = as.numeric(unlist(xpathApply(aNode, ".//RangeGate", xmlGetAttr, "min")));
          rangeMaxes = as.numeric(unlist(xpathApply(aNode, ".//RangeGate", xmlGetAttr, "max")));
        }
      }
      
      # STRIP OUT THE "Comp-" PART OF NAMES.   THE TRANSFORMATION IS ONLY ON THE VISUALIZATION
      # OF DATA IN FLOWJO GRAPHS AND HAS NO IMPACT ON THE STORED GATING COORDINATES.
      if (stripComp == TRUE) {
        rangeNames = gsub("\\bComp-", "", rangeNames, perl = TRUE)
      }
      
      verticies = matrix(c(rangeMins, rangeMaxes), ncol = length(rangeNames),
          nrow = 2, byrow = TRUE);
      colnames(verticies) = rangeNames;
      for (i in 1:length(rangeNames)) { 
        if (length(grep("FSC-[AHW]|SSC-[AHW]", rangeNames[i])) > 0) {
          verticies[,i] = verticies[,i] * multiplier
        }
      }
      
      thisGate = rectangleGate(filterId = gateName, .gate=verticies);
      thisGate@filterId = gateName
      return(thisGate)
    }
    
    # ELLIPSOID GATES -- SKIP FOR NOW
    if (gateType == "EllipseGate") {
        if (quiet == FALSE) {
            print("Sorry, I haven't worked out EllipseGates yet.")
        }
        return(NULL)
    }
    
    # NO IDEA WHAT TYPE OF NEWFANGLED GATE THIS IS...
    cat(paste("Sorry, I don't know how to handle", gateType, "type gates\n"))
    return(NULL)
});



