setMethod("getFlowJoCompMatrix",
          signature(flowJoDOM = "XMLInternalDocument"), 
          definition = function(flowJoDOM) {

  sampleXpath  <- "/Workspace/SampleList/Sample";
  sampleNodes  <- xpathApply(flowJoDOM, path=sampleXpath, xmlChildren);
  fjVersion    <- getFlowJoVersion(flowJoDOM);
  fsep         <- .Platform$file.sep;
  compList     <- list();

  
  # This loops over the samples in your Workspace.  There can theoretically be
  # as many different compensation matrices as there are samples.  Though in
  # practice this doesn't happen very often.  Under each "Sample" node are
  # 'Keywords', 'DataSet', 'CompensationMatrix' and 'SampleNode'
  for (i in 1:length(sampleNodes)) {
    assemblyList   <- list();
    aSampleNode    <- sampleNodes[[i]];

    # Get the name of the FCS file as 'sampleName'
    sampleName     <- xmlGetAttr(aSampleNode$DataSet, "uri"); 
    sampleName     <- gsub("\\\\|\\/", fsep, sampleName, perl=TRUE);
    sampleName     <- basename(sampleName); 
        
    # Get the 'CompensationMatrix' for this FCS file.  Here, the earlier and more
    # recent versions of FlowJo diverge in their XML structure.
    # FIRST, the earlier versions of the spillover matrices
    if (compareVersion(fjVersion, "7.4.9") < 0) {
      aSpillNode  <- aSampleNode$CompensationMatrix
      if (length(aSpillNode) > 0) {
        for (aChannel in xmlChildren(aSpillNode)) {
          channelName   = xmlGetAttr(aChannel, "name")
          overlapNames  = as.character(xmlApply(aChannel, xmlGetAttr, "name"))
          overlapValues = as.numeric(xmlApply(aChannel, xmlGetAttr, "value"))
          for (i in 1:length(overlapNames))  assemblyList[[channelName]][[ overlapNames[i] ]] = overlapValues[i]
        }
      }

    # THEN, the more recent versions of the spillover matrices      
    } else {  
      aSpillNode  <- aSampleNode$spilloverMatrix
      if (length(aSpillNode) > 0) {
        for (aChannel in xmlChildren(aSpillNode)) {
          channelName   = xmlGetAttr(aChannel, "parameter")
          overlapNames  = as.character(xmlApply(aChannel, xmlGetAttr, "parameter"))
          overlapValues = as.numeric(xmlApply(aChannel, xmlGetAttr, "value"))
          for (i in 1:length(overlapNames))  assemblyList[[channelName]][[ overlapNames[i] ]] = overlapValues[i]
        }
      }
    }


    # Then convert the "list" into a "matrix".  Note, I could have done this
    # earlier but I'm not sure that the order of the channel names will be
    # the same in each "Channel" node.   Will also return a 0 X 0 matrix in 
    # the case that there is no spillover matrix for this file.
    numChannels     = length(assemblyList);
    channelNamesOrd = names(assemblyList);
    assemblyMat     = matrix(nrow=numChannels, ncol=numChannels, byrow=FALSE,
                          dimnames=list(channelNamesOrd, channelNamesOrd));
                      
    for (topChannel in channelNamesOrd) {
      for (leftChannel in channelNamesOrd) {
        assemblyMat[leftChannel, topChannel] = as.numeric(assemblyList[[leftChannel]][[topChannel]]);
      }
    }
        
    # Note that what finally gets returned is a list of compensation matricies
    compList[[sampleName]] = assemblyMat;
  }

  return(compList);
});




