setMethod("readFlowJoFile",
          signature(xmlFile = "character"), 
          definition = function(xmlFile) {
  
    ## Check whether the FlowJo workspace exists.  
    if (! file.exists(xmlFile)) {
      stop(paste("I don't see a file at:", xmlFile)); 
    }
     
    flowJoDOM <- xmlTreeParse(xmlFile, useInternalNodes=TRUE,
                              error = xmlErrorCumulator(class = "XMLParserErrorList",
                                                        immediate = FALSE))
    
    ## XML for versions 7.5.0 --> 7.6 are malformed and
    ## namespace declaration needed to be inserted into the data.
    ## detect, insert and re-read:
    
    # There are apparently many ways that the FlowJo version is named such as:
    # "Version 7.2.2", "Version 7.2.2 beta 13", "7.2.2beta13", "7.44.2beta 13", or "v7.2.2beta13" 
    # Made a minor improvement to 'getFlowJoVersion' to handle these alternatives
#    flowJoVersion <- sub('[^\\d\\.].*?$', '', flowJoVersion, perl=TRUE)
    flowJoVersion <- getFlowJoVersion(flowJoDOM)

    # The namespace attribute is special and not in xmlAttrs()
    ns.missing <- is.null(xmlNamespaces(xmlRoot(flowJoDOM)))

    if ((compareVersion(flowJoVersion, "7.4.9") > 0) && (ns.missing)) {
      cat("found 7.5+ wsp without namespace...")
      fj.encoding <- getEncoding(flowJoDOM)
      free(flowJoDOM)
      # Most FlowJo workspace are latin1/iso8859-1, not utf-8,
      # setting encoding explicitly:
      xmlText <- scan(file=xmlFile, what="character", sep="\n", quiet=TRUE, encoding=fj.encoding)
      xmlText <- paste(xmlText, collapse="\n")

      nameSpaceDefinitions <- c('xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"',
                                'xmlns:gating="http://www.isac-net.org/std/Gating-ML/v1.5/gating"',
                                'xmlns:transforms="http://www.isac-net.org/std/Gating-ML/v1.5/transformations"',
                                'xmlns:data-type="http://www.isac-net.org/std/Gating-ML/v1.5/datatypes"',
                                'xsi:schemaLocation="http://www.isac-net.org/std/Gating-ML/v1.5/gating', # xsi begin
                                'http://www.isac-net.org/std/Gating-ML/v1.5/gating',
                                'http://www.isac-net.org/std/Gating-ML/v1.5/gating/Gating-ML.v1.5.xsd',
                                'http://www.isac-net.org/std/Gating-ML/v1.5/transformations',
                                'http://www.isac-net.org/std/Gating-ML/v1.5/gating/Transformations.v1.5.xsd',
                                'http://www.isac-net.org/std/Gating-ML/v1.5/datatypes',
                                'http://www.isac-net.org/std/Gating-ML/v1.5/gating/DataTypes.v1.5.xsd"') # xsi end

      nameSpacedWorkspace <- paste(c("<Workspace", nameSpaceDefinitions), collapse=" ")
      xmlText <- sub("<Workspace", nameSpacedWorkspace, xmlText)
      flowJoDOM <- xmlTreeParse(xmlText, useInternalNodes=TRUE,
                                error = xmlErrorCumulator(class = "XMLParserErrorList",
                                  immediate = FALSE))
      #cat(xmlText, file=paste(wspLocations, "fixed.wsp", sep=''))
    }
    docName(flowJoDOM) <- xmlFile
  
  
    ## Get the root of the DOM tree; expect it to be a "Workspace" node
    root <- xmlRoot(flowJoDOM);
    if(xmlName(root) != "Workspace") {
      stop("Top node is not a 'Workspace' node.  Is this really a FlowJo workspace?!?");
    }
  
    ## This Workspace should refer to at least one or more FCS files
    numSamples = length(xpathApply(flowJoDOM,
                                   path="/Workspace/SampleList/Sample/DataSet",
                                   xmlChildren));
                                   
    if (numSamples < 1) {
      warning("This workspace doesn't refer to any FCS files!");
    }

    numGates = length(xpathApply(flowJoDOM,
                                 path="/Workspace/SampleList/Sample/SampleNode/Subpopulations",
                                 xmlChildren));
                                   
    if (numGates < 1) {
      warning("This workspace doesn't contain any gates!");
    }
    
    return(flowJoDOM);            
});
