\name{createFlowReport}
\alias{createFlowReport}
\alias{createFlowReport,fcsSummaryList-method}

\title{ Produce a denormalized data frame of summary statistics and other information on a set of FlowJo gated FCS files }
\description{
  This method produces a data frame that has one row per cell population per FCS file per channel for one or more FCS files as gated using the commercial software, FlowJo.
}
\usage{
createFlowReport(summaryList, \dots)
}

\arguments{
  \item{summaryList}{ a flowFlowJo object that is a list of data structures providing many bits of summary information about each gated population in a gated FCS file.  See also \emph{collectSummaryFlowInfo} }
  \item{\dots}{  Additional factors detailed below.}
}  
 
\details{
  Additional arguments also include the following:  
  \preformatted{
    factorsFrame    - an additional data frame with various types of meta data related to the experiment in question 
    populations     - a list of cell population names to be reported on.  By default, the summary statistics on all the cell populations will be reported. 
    channels        - a list of scatter and non-scatter (fluorescent) channels to be reported on.  By default, the summary statistics for all channels will be reported. 
    includeKeywords - a boolean with the default = TRUE.  If FALSE, then none of the keywords from the headers of the FCS files that were harvested in an earlier step will be reported. 
    joinFactors     - if the user supplies an optional data frame with additional meta data about the experiment, these columns in the data frame will be used to match up items in the data frame with the correct FCS files. The default is joinFactors=c("FCSFilename", "FlowJoWorkspace")
    dropJoinFactors - a boolean with default = TRUE.  If set to FALSE, then the name (and path) of the FCS file and the FlowJo workspace will be included in every  line of the report.  That would make the report very ugly to read.   
  }
}


\value{
The \emph{collectSummaryFlowInfo} method produces a large and complicated data structure that tracks every FCS file location, gate structure, compensation matrix, as well as the summary statistics for every file referenced in a FlowJo workspace.  This method, \emph{createFlowReport} distills out a simple data frame of the summary statistics that is convenient and easy to read, plot, and otherwise analyze. 
}

\author{ John Gosink }

\examples{

library(flowFlowJo);
demoLocation <- system.file("extdata", "DemoWorkspace.wsp", package="flowFlowJo"); 
actualFCSLoc <- system.file("extdata/fcsFiles", package="flowFlowJo"); 
testList     <- readFlowJoList(demoLocation, altFileLocation=actualFCSLoc);

# This next statement may take a few moments to run as it is parsing through a FlowJo workspace
# and two dozen FCS files...
summaryStatsObj <- collectSummaryFlowInfo(testList);

# And here are some of the summary statistics for this workspace
flowReport      <- createFlowReport(summaryStatsObj);
head(flowReport);

}

\keyword{methods}
\keyword{list}

