\name{flowFlowJo-package}
\alias{flowFlowJo-package}
\alias{flowFlowJo}
\docType{package}
\title{
Turn FlowJo workspaces into flowCore compliant objects
}
\description{
Extract information, gates, FCS file (locations) etc. from a FlowJo workspace

}
\details{
\tabular{ll}{
Package: \tab flowFlowJo\cr
Type: \tab Package\cr
Version: \tab 1.16\cr
Date: \tab 2008-09-25\cr
License: \tab GPL 3 or newer\cr
}

FlowJo is a commercially available software package used in gating and analyzing data from flow cytometry experiments.  Upon saving, FlowJo produces an XML format file that contains all the information necessary to describe the gating structures, compensation, transformation, location of the FCS files, and opened graphs and figures last created by the user.  This package (flowFlowJo) is a small set of methods designed to extract the file locations, gates, compensation matrices and other meta-data contained in FlowJo workspaces and return the information in a manner consistent with the BioConductor flowCore packages.

}

\author{
John Gosink
<gosinkj@amgen.com>
}

\note{ See a complete discussion with examples via: \emph{vignette("flowFlowJo")} }


\examples{

# A demo set of FCS files and attendant FlowJo workspace are included with
# this package.  This extracts the info from the workspace.

library(flowFlowJo);
demoLocation <- system.file("extdata", "DemoWorkspace.wsp", package="flowFlowJo"); 
actualFCSLoc <- system.file("extdata/fcsFiles", package="flowFlowJo"); 
testList     <- readFlowJoList(demoLocation, altFileLocation=actualFCSLoc);
z            <- getFlowJoGates(testList, fileNamePatterns=c("C02"));

# Take a look at z.  It contains the name(s), location(s), compensation matrix, gate/filter(s) 
# and gate/filter names for one of the FCS files referenced in the workspace.

}